;;;
;;;Part of: Vicare Libesmtp
;;;Contents: libesmtp constants
;;;Date: Thu Feb 14, 2013
;;;
;;;Abstract
;;;
;;;	Export an identifier syntax for each Libesmtp constant.
;;;
;;;Copyright (C) 2013, 2015 Marco Maggi <marco.maggi-ipsu@poste.it>
;;;
;;;This program is free software:  you can redistribute it and/or modify
;;;it under the terms of the  GNU General Public License as published by
;;;the Free Software Foundation, either version 3 of the License, or (at
;;;your option) any later version.
;;;
;;;This program is  distributed in the hope that it  will be useful, but
;;;WITHOUT  ANY   WARRANTY;  without   even  the  implied   warranty  of
;;;MERCHANTABILITY  or FITNESS FOR  A PARTICULAR  PURPOSE.  See  the GNU
;;;General Public License for more details.
;;;
;;;You should  have received  a copy of  the GNU General  Public License
;;;along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;;


(library (vicare email libesmtp constants)
  (export

    ;; enum header_option
    Hdr_OVERRIDE
    Hdr_PROHIBIT

    ;; protocol progress
    SMTP_EV_CONNECT
    SMTP_EV_MAILSTATUS
    SMTP_EV_RCPTSTATUS
    SMTP_EV_MESSAGEDATA
    SMTP_EV_MESSAGESENT
    SMTP_EV_DISCONNECT
    SMTP_EV_SYNTAXWARNING

    ;; protocol extension progress
    SMTP_EV_ETRNSTATUS

    ;; required extensions
    SMTP_EV_EXTNA_DSN
    SMTP_EV_EXTNA_8BITMIME
    SMTP_EV_EXTNA_STARTTLS
    SMTP_EV_EXTNA_ETRN
    SMTP_EV_EXTNA_CHUNKING
    SMTP_EV_EXTNA_BINARYMIME

    ;; Extensions specific events
    SMTP_EV_DELIVERBY_EXPIRED

    ;; STARTTLS
    SMTP_EV_WEAK_CIPHER
    SMTP_EV_STARTTLS_OK
    SMTP_EV_INVALID_PEER_CERTIFICATE
    SMTP_EV_NO_PEER_CERTIFICATE
    SMTP_EV_WRONG_PEER_CERTIFICATE
    SMTP_EV_NO_CLIENT_CERTIFICATE
    SMTP_EV_UNUSABLE_CLIENT_CERTIFICATE
    SMTP_EV_UNUSABLE_CA_LIST

    ;; enum rfc2822_timeouts
    Timeout_GREETING
    Timeout_ENVELOPE
    Timeout_DATA
    Timeout_TRANSFER
    Timeout_DATA2
    Timeout_OVERRIDE_RFC2822_MINIMUM

    ;; enum notify_flags
    Notify_NOTSET
    Notify_NEVER
    Notify_SUCCESS
    Notify_FAILURE
    Notify_DELAY

    ;; enum e8bitmime_body
    E8bitmime_NOTSET
    E8bitmime_7BIT
    E8bitmime_8BITMIME
    E8bitmime_BINARYMIME

    ;; enum by_mode
    By_NOTSET
    By_NOTIFY
    By_RETURN

    ;; enum starttls_option
    Starttls_DISABLED
    Starttls_ENABLED
    Starttls_REQUIRED

    ;;
    SMTP_ERR_NOTHING_TO_DO
    SMTP_ERR_DROPPED_CONNECTION
    SMTP_ERR_INVALID_RESPONSE_SYNTAX
    SMTP_ERR_STATUS_MISMATCH
    SMTP_ERR_INVALID_RESPONSE_STATUS
    SMTP_ERR_INVAL
    SMTP_ERR_EXTENSION_NOT_AVAILABLE

    ;; enum ret_flags
    Ret_NOTSET
    Ret_FULL
    Ret_HDRS

    ;; Deprecated - these will be removed in a future release
    SMTP_ERR_HOST_NOT_FOUND
    SMTP_ERR_NO_ADDRESS
    SMTP_ERR_NO_RECOVERY
    SMTP_ERR_TRY_AGAIN

    ;; libESMTP versions of some getaddrinfo error numbers
    SMTP_ERR_EAI_AGAIN
    SMTP_ERR_EAI_FAIL
    SMTP_ERR_EAI_MEMORY
    SMTP_ERR_EAI_ADDRFAMILY
    SMTP_ERR_EAI_NODATA
    SMTP_ERR_EAI_FAMILY
    SMTP_ERR_EAI_BADFLAGS
    SMTP_ERR_EAI_NONAME
    SMTP_ERR_EAI_SERVICE
    SMTP_ERR_EAI_SOCKTYPE
    SMTP_ERR_UNTERMINATED_RESPONSE
    SMTP_ERR_CLIENT_ERROR

    ;; Protocol monitor callback.  Values for writing
    SMTP_CB_READING
    SMTP_CB_WRITING
    SMTP_CB_HEADERS

;;; --------------------------------------------------------------------

    AUTH_USER
    AUTH_REALM
    AUTH_PASS
    AUTH_CLEARTEXT

    AUTH_PLUGIN_PLAIN
    AUTH_PLUGIN_EXTERNAL)
  (import (rename (only (vicare)
			define-inline-constant)
		  (define-inline-constant	defc)))


;;;; constants

;; enum header_option
(defc Hdr_OVERRIDE		0)
(defc Hdr_PROHIBIT		1)

;; protocol progress
(defc SMTP_EV_CONNECT		0)
(defc SMTP_EV_MAILSTATUS	1)
(defc SMTP_EV_RCPTSTATUS	2)
(defc SMTP_EV_MESSAGEDATA	3)
(defc SMTP_EV_MESSAGESENT	4)
(defc SMTP_EV_DISCONNECT	5)
(defc SMTP_EV_SYNTAXWARNING	6)

;; protocol extension progress
(defc SMTP_EV_ETRNSTATUS	1000)

;; required extensions
(defc SMTP_EV_EXTNA_DSN		2000)
(defc SMTP_EV_EXTNA_8BITMIME	2001)
(defc SMTP_EV_EXTNA_STARTTLS	2002)
(defc SMTP_EV_EXTNA_ETRN	2003)
(defc SMTP_EV_EXTNA_CHUNKING	2004)
(defc SMTP_EV_EXTNA_BINARYMIME	2005)

;; Extensions specific events
(defc SMTP_EV_DELIVERBY_EXPIRED	3000)

;; STARTTLS
(defc SMTP_EV_WEAK_CIPHER	3100)
(defc SMTP_EV_STARTTLS_OK	3101)
(defc SMTP_EV_INVALID_PEER_CERTIFICATE		3102)
(defc SMTP_EV_NO_PEER_CERTIFICATE		3103)
(defc SMTP_EV_WRONG_PEER_CERTIFICATE		3104)
(defc SMTP_EV_NO_CLIENT_CERTIFICATE		3105)
(defc SMTP_EV_UNUSABLE_CLIENT_CERTIFICATE	3106)
(defc SMTP_EV_UNUSABLE_CA_LIST	3107)

;; enum rfc2822_timeouts
(defc Timeout_GREETING		0)
(defc Timeout_ENVELOPE		1)
(defc Timeout_DATA		2)
(defc Timeout_TRANSFER		3)
(defc Timeout_DATA2		4)
(defc Timeout_OVERRIDE_RFC2822_MINIMUM		4096)

;; enum notify_flags
(defc Notify_NOTSET		0)
(defc Notify_NEVER		-1)
(defc Notify_SUCCESS		1)
(defc Notify_FAILURE		2)
(defc Notify_DELAY		4)

;; enum e8bitmime_body
(defc E8bitmime_NOTSET		0)
(defc E8bitmime_7BIT		1)
(defc E8bitmime_8BITMIME	2)
(defc E8bitmime_BINARYMIME	3)

;; enum by_mode
(defc By_NOTSET			0)
(defc By_NOTIFY			1)
(defc By_RETURN			2)

;; enum starttls_option
(defc Starttls_DISABLED		0)
(defc Starttls_ENABLED		1)
(defc Starttls_REQUIRED		2)

;;
(defc SMTP_ERR_NOTHING_TO_DO		2)
(defc SMTP_ERR_DROPPED_CONNECTION	3)
(defc SMTP_ERR_INVALID_RESPONSE_SYNTAX	4)
(defc SMTP_ERR_STATUS_MISMATCH		5)
(defc SMTP_ERR_INVALID_RESPONSE_STATUS	6)
(defc SMTP_ERR_INVAL			7)
(defc SMTP_ERR_EXTENSION_NOT_AVAILABLE	8)

;; enum ret_flags
(defc Ret_NOTSET		0)
(defc Ret_FULL			1)
(defc Ret_HDRS			2)

;; Deprecated - these will be removed in a future release
(defc SMTP_ERR_HOST_NOT_FOUND	#f)
(defc SMTP_ERR_NO_ADDRESS	#f)
(defc SMTP_ERR_NO_RECOVERY	#f)
(defc SMTP_ERR_TRY_AGAIN	#f)

;; libESMTP versions of some getaddrinfo error numbers
(defc SMTP_ERR_EAI_AGAIN	12)
(defc SMTP_ERR_EAI_FAIL		11)
(defc SMTP_ERR_EAI_MEMORY	13)
(defc SMTP_ERR_EAI_ADDRFAMILY	9)
(defc SMTP_ERR_EAI_NODATA	10)
(defc SMTP_ERR_EAI_FAMILY	14)
(defc SMTP_ERR_EAI_BADFLAGS	15)
(defc SMTP_ERR_EAI_NONAME	16)
(defc SMTP_ERR_EAI_SERVICE	17)
(defc SMTP_ERR_EAI_SOCKTYPE	18)
(defc SMTP_ERR_UNTERMINATED_RESPONSE	19)
(defc SMTP_ERR_CLIENT_ERROR	20)

;; Protocol monitor callback.  Values for writing
(defc SMTP_CB_READING		0)
(defc SMTP_CB_WRITING		1)
(defc SMTP_CB_HEADERS		2)

;;; --------------------------------------------------------------------

(defc AUTH_USER			1)
(defc AUTH_REALM		2)
(defc AUTH_PASS			4)
(defc AUTH_CLEARTEXT		8)

(defc AUTH_PLUGIN_PLAIN		2)
(defc AUTH_PLUGIN_EXTERNAL	4)


;;;; done

)

;;; end of file
;; Local Variables:
;; mode: scheme
;; End:
